//data storage variables
byte newData = 0;
byte prevData = 0;
unsigned int time = 0;//keeps time and sends vales to store in timer[] occasionally
int timer[10];//storage for timing of events
int slope[10];//storage for slope of events
unsigned int totalTimer;//used to calculate period
unsigned int period;//storage for period of wave
byte index = 0;//current storage index
float frequency;//storage for frequency calculations
int maxSlope = 0;//used to calculate max slope as trigger point
int newSlope;//storage for incoming slope data

//variables for deciding whether you have a match
byte noMatch = 0;//counts how many non-matches you've received to reset variables if it's been too long
byte slopeTol = 3;//slope tolerance- adjust this if you need
int timerTol = 10;//timer tolerance- adjust this if you need

//variables for amp detection
unsigned int ampTimer = 0;
byte maxAmp = 0;
byte checkMaxAmp;
byte ampThreshold = 32;//raise if you have a very noisy signal

int selectedNote = 0; // index corresponding to array below
float correctFrequency[] = {82.4, 110, 146.8, 196, 246.9, 329.6}; // [E, A, D, G, B, e] standard tuning
#define E 0
#define A 1
#define D 2
#define G 3
#define B 4
#define e 5

#define ENABLE 9
#define DIR1 8
#define DIR2 7

#define BUTTON 12

void setup(){
  Serial.begin(9600);

  //setting up L293D H bridge for driving the motor
  pinMode(ENABLE, OUTPUT);
  digitalWrite(ENABLE, HIGH);
  pinMode(DIR1, OUTPUT);
  pinMode(DIR2, OUTPUT);

  pinMode(BUTTON, INPUT);
  // no need for pullup resistor now
  digitalWrite(BUTTON, HIGH); 
  
  cli();//disable interrupts
  
  //set up continuous sampling of analog pin 0 at 38.5kHz
 
  //clear ADCSRA and ADCSRB registers
  ADCSRA = 0;
  ADCSRB = 0;
  
  ADMUX |= (1 << REFS0); //set reference voltage
  ADMUX |= (1 << ADLAR); //left align the ADC value- so we can read highest 8 bits from ADCH register only
  
  ADCSRA |= (1 << ADPS2) | (1 << ADPS0); //set ADC clock with 32 prescaler- 16mHz/32=500kHz
  ADCSRA |= (1 << ADATE); //enabble auto trigger
  ADCSRA |= (1 << ADIE); //enable interrupts when measurement complete
  ADCSRA |= (1 << ADEN); //enable ADC
  ADCSRA |= (1 << ADSC); //start ADC measurements
  
  sei();//enable interrupts
}

ISR(ADC_vect) {//when new ADC value ready
  
  PORTB &= B11101111;//set pin 12 low
  prevData = newData;//store previous value
  newData = ADCH;//get value from A0
  if (prevData < 127 && newData >=127){//if increasing and crossing midpoint
    newSlope = newData - prevData;//calculate slope
    if (abs(newSlope-maxSlope)<slopeTol){//if slopes are ==
      //record new data and reset time
      slope[index] = newSlope;
      timer[index] = time;
      time = 0;
      if (index == 0){//new max slope just reset
        PORTB |= B00010000;//set pin 12 high
        noMatch = 0;
        index++;//increment index
      }
      else if (abs(timer[0]-timer[index])<timerTol && abs(slope[0]-newSlope)<slopeTol){//if timer duration and slopes match
        //sum timer values
        totalTimer = 0;
        for (byte i=0;i<index;i++){
          totalTimer+=timer[i];
        }
        period = totalTimer;//set period
        //reset new zero index values to compare with
        timer[0] = timer[index];
        slope[0] = slope[index];
        index = 1;//set index to 1
        PORTB |= B00010000;//set pin 12 high
        noMatch = 0;
      }
      else{//crossing midpoint but not match
        index++;//increment index
        if (index > 9){
          reset();
        }
      }
    }
    else if (newSlope>maxSlope){//if new slope is much larger than max slope
      maxSlope = newSlope;
      time = 0;//reset clock
      noMatch = 0;
      index = 0;//reset index
    }
    else{//slope not steep enough
      noMatch++;//increment no match counter
      if (noMatch>9){
        reset();
      }
    }
  }
  
  time++;//increment timer at rate of 38.5kHz
  
  ampTimer++;//increment amplitude timer
  if (abs(127-ADCH)>maxAmp){
    maxAmp = abs(127-ADCH);
  }
  if (ampTimer==1000){
    ampTimer = 0;
    checkMaxAmp = maxAmp;
    maxAmp = 0;
  }
  
}

void reset(){//clean out some variables
  index = 0;//reset index
  noMatch = 0;//reset match couner
  maxSlope = 0;//reset slope
}

//Drive motor in the specified direction for 100 ms
void driveMotor(int dir) {
  digitalWrite(dir, HIGH);
  delay(100);
  digitalWrite(dir,LOW);
}

//Compare the frequency input to the correct frequency
void frequencyCheck(){
  if(frequency>correctFrequency[selectedNote]+5){
    Serial.println("Too HIGH");
    driveMotor(DIR1);
  }
  if(frequency<correctFrequency[selectedNote]-5){
    Serial.println("Too LOW");
    driveMotor(DIR2);
  }
  if((frequency>correctFrequency[selectedNote]-5) && (frequency<correctFrequency[selectedNote]+5)){
    Serial.println("Tuned");
  }
}
long ts = 0;
int buttonState = 1, lastButtonState = 1;
//Select note by button press
void selectNote() {
  int reading = digitalRead(BUTTON);

  if (reading != lastButtonState) {
    // reset the debouncing timer
    ts = millis();
  }
  
  if ((millis() - ts) >= 100) {
      // my non-blocking timed loop here
      if (reading != buttonState) {
        buttonState = reading;
        if(reading == 0) {
          if (selectedNote < 5) {
            selectedNote++;
          } else selectedNote = 0;
          reading = 1;
        Serial.println(selectedNote);
        }
      }
  }
  
  lastButtonState = reading;
}

void loop(){ 
  selectNote();
  
  //Serial.println(checkMaxAmp);
  if (checkMaxAmp>ampThreshold){
    frequency = 38462/float(period);//calculate frequency timer rate/period
    Serial.println(frequency);
  }
  //frequencyCheck();
  
  delay(100);
 
}
